/*
 * XClipboardPlugin.java
 * Copyright (C) 2000 mike dillon
 *               
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.Component;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EBPlugin;
import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.JARClassLoader;
import org.gjt.sp.jedit.MiscUtilities;
import org.gjt.sp.jedit.View;

import org.gjt.sp.jedit.msg.EditPaneUpdate;

import org.gjt.sp.jedit.textarea.JEditTextArea;

import org.gjt.sp.util.Log;

import util.nmeyers.XClipboard;

public class XClipboardPlugin extends EBPlugin
	implements MouseListener
{
	private static final XClipboard clipboard;
	static
	{
		try
		{
			// Determine the location of the JAR file
			JARClassLoader cLoader = (JARClassLoader)
                                XClipboardPlugin.class.getClassLoader();

			// XXX: this is a hack to figure out the JAR path,
			// since Slava removed the API
			String jarIdxStr = cLoader.getResourceAsPath("")
				.substring("jeditresource:".length());
			String jarPath = jEdit.getPluginJAR(Integer.parseInt(
				jarIdxStr.substring(0, jarIdxStr.length() - 1)))
				.getPath();

			// Construct a path for the SO in the JAR's directory
			String libPath = MiscUtilities.constructPath(
				MiscUtilities.getFileParent(jarPath),
				"libXClipboard.so");

			Log.log(Log.MESSAGE, XClipboardPlugin.class,
				"Loading shared library: " + libPath);

			// Load the shared library for the native functions
			System.load(libPath);
		}
		catch (UnsatisfiedLinkError ule)
		{
			// give the user a slight hint
			Log.log(Log.ERROR, XClipboardPlugin.class,
				"Missing required native library. "
                                + "See README.Plugin.");

			// we want the class loading to fail but we don't want
			// to log the exception twice, so throw it and let
			// JARClassLoader log it
			throw ule;
		}

		clipboard = new XClipboard();
	}

	public void handleMessage(EBMessage message)
	{
		if(message instanceof EditPaneUpdate)
		{
			EditPaneUpdate eu = (EditPaneUpdate) message;

			if(eu.getWhat() == EditPaneUpdate.CREATED)
			{
				eu.getEditPane().getTextArea().getPainter().addMouseListener(this);
			}
			else if(eu.getWhat() == EditPaneUpdate.DESTROYED)
			{
				eu.getEditPane().getTextArea().getPainter().removeMouseListener(this);
			}
		}
	}

	public void mousePressed(MouseEvent evt)
	{
		if((evt.getModifiers() & MouseEvent.BUTTON2_MASK) == 0) return;

		String selection = clipboard.readPrimarySelectionString();

		if (selection == null) return;

		// the event source should be a TextAreaPainter, whose parent is
		// always a JEditTextArea
		JEditTextArea textArea =
			(JEditTextArea) ((Component)evt.getSource()).getParent();

		textArea.setCaretPosition(textArea.xyToOffset(evt.getX(),
			evt.getY()));

		textArea.setSelectedText(selection);

		evt.consume();
	}

	public void mouseClicked(MouseEvent evt) {}
	public void mouseEntered(MouseEvent evt) {}
	public void mouseExited(MouseEvent evt) {}
	public void mouseReleased(MouseEvent evt) {}
}
